""" Record the process of algorithms in the log file """


from mpi4py import MPI

comm = MPI.COMM_WORLD
size = MPI.COMM_WORLD.Get_size()
rank = MPI.COMM_WORLD.Get_rank()

class Log(object):

    def __init__(self):
        self.alg_name = None
        self.data_shape = None
        self.start_time = None
        self.end_time = None
        self.end_iter = 0
        self.wall_clock = 0
        self.size = size
        self.step_size = None
        self.penalty_param = 0
        self.consensus_step = 0
        self.function_value = 0
        self.consensus_error = 1
        self.kkt_violation = 1
        self.feasibility_violation = 1
        self.iter_history = []
        self.time_history = []
        self.step_size_history = []
        self.function_value_history = []
        self.kkt_violation_history = []
        self.feasibility_violation_history = []
        self.consensus_error_history = []
        self.graph = None

    def record(self, alg_name = None, data_shape = None, start_time = None, iter = None, wall_clock = None, 
                size = None, consensus_step = None, step_size = None, penalty_param = None,
                function_value = None, consensus_error = None, kkt_violation = None, feasibility_violation = None, graph = None):
        """ record the variables, grad norm, function value with an iteration and time. """

        if alg_name is not None:
            self.alg_name = alg_name

        if data_shape is not None:
            self.data_shape = data_shape

        if size is not None:
            self.size = size

        if start_time is not None:
            self.start_time = start_time

        if iter is not None:
            if self.start_time is None:
                self.start_time = MPI.Wtime()
            time = MPI.Wtime() - self.start_time
            self.iter_history.append(iter)
            self.time_history.append(time)
            self.end_time = time
            self.end_iter = iter

        if wall_clock is not None:
            self.wall_clock = wall_clock

        if step_size is not None:
            self.step_size = step_size
            step_size_grad = self.step_size.get('gradient')
            self.step_size_history.append(step_size_grad)

        if penalty_param is not None:
            self.penalty_param = penalty_param

        if consensus_step is not None:
            self.consensus_step = consensus_step

        if function_value is not None:
            self.function_value = function_value
            self.function_value_history.append(function_value)

        if consensus_error is not None:
            self.consensus_error = consensus_error
            self.consensus_error_history.append(consensus_error)

        if kkt_violation is not None:
            self.kkt_violation = kkt_violation
            self.kkt_violation_history.append(kkt_violation)

        if feasibility_violation is not None:
            self.feasibility_violation = feasibility_violation
            self.feasibility_violation_history.append(feasibility_violation)

        if graph is not None:
            self.graph = graph

