import pickle
import matplotlib.pyplot as plt
import matplotlib.colors as colors

import os, sys 
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__))) 
sys.path.insert(0, parentdir)

line_stype = ['-2', '-*', '-+', '-d', '-x', '-1', '-v']


def plot_kkt_violation(log, curve_type, color, size = 10, interval = 15, linewidth = 3):

    alg_name = log.alg_name
    step_size = log.step_size
    if step_size is not None:
        step_size_cons = step_size.get('consensus')
        step_size_grad = step_size.get('gradient')
        step_size_type = step_size.get('type')
        if step_size_type == 'BB':
            label = alg_name + ', ' + r'$\eta = \eta^{\mathrm{BB}}_{i, k}$'
        else:
            label = alg_name + ', ' + r'$\eta = $' + str(step_size_grad)

    else:
        label = alg_name

    plt.figure(1)
    plt.plot(log.kkt_violation_history, curve_type, c = color, linewidth = linewidth, 
                markersize = size, markevery = len(log.kkt_violation_history) // interval,
                label = label)
    plt.yscale('log')
    plt.ylabel('Substationarity Violation', fontsize = 15)
    plt.xlabel('Round of Communications', fontsize = 15)

def plot_consensus_error(log, curve_type, color, size = 10, interval = 15, linewidth = 3):

    alg_name = log.alg_name
    step_size = log.step_size
    if step_size is not None:
        step_size_cons = step_size.get('consensus')
        step_size_grad = step_size.get('gradient')
        step_size_type = step_size.get('type')
        if step_size_type == 'BB':
            label = alg_name + ', ' + r'$\eta = \eta^{\mathrm{BB}}_{i, k}$'
        else:
            label = alg_name + ', ' + r'$\eta = $' + str(step_size_grad)

    else:
        label = alg_name

    plt.figure(2)
    plt.plot(log.consensus_error_history, curve_type, c = color, linewidth = linewidth, 
                markersize = size, markevery = len(log.kkt_violation_history) // interval,
                label = label)
    plt.yscale('log')
    plt.ylabel('Consensus Error', fontsize = 15)
    plt.xlabel('Round of Communications', fontsize = 15)


files_name = 'Results/Pkls'


################################################################################################################################################

""" KKT violation """
save_file_name = os.path.join(files_name, 'DRGTA.pkl')
with open(save_file_name, 'rb') as input:
    log = pickle.load(input)
    curve_type = line_stype[0]
    color = 'orange'
    plot_kkt_violation(log, curve_type, color = color, size = 10, interval = 15, linewidth = 3)

save_file_name = os.path.join(files_name, 'DESTINY.pkl')
with open(save_file_name, 'rb') as input:
    log = pickle.load(input)
    curve_type = line_stype[1]
    color = 'royalblue'
    plot_kkt_violation(log, curve_type, color = color, size = 10, interval = 15, linewidth = 3)


large_font = 10
plt.rc('xtick', labelsize = 20)    # fontsize of the x and y labels
plt.xticks(fontsize = large_font)
plt.rc('ytick', labelsize = 20)
plt.yticks(fontsize = large_font)

plt.rc('legend', fontsize = 15)    # legend fontsize

plt.legend()
#plt.legend(loc = 'right')

graph_type, weighted_rule, prob = log.graph

if not os.path.isdir("Results/Figures"):
    os.makedirs("Results/Figures")
save_path = 'Results/Figures'
#filename = 'PCA_shape_' + str(log.data_shape) + '_nodes_' + str(log.size) + '_p_' + str(prob)+ '_'+ graph_type +'_'+ weighted_rule  +'.pdf'
filename = 'Test_DESTINY_kkt.pdf'
plt.tight_layout()
plt.savefig(os.path.join(save_path, filename), format = 'pdf', dpi = 3000)
plt.show()


################################################################################################################################################

plt.clf()

""" consensus error """
save_file_name = os.path.join(files_name, 'DRGTA.pkl')
with open(save_file_name, 'rb') as input:
    log = pickle.load(input)
    curve_type = line_stype[0]
    color = 'orange'
    plot_consensus_error(log, curve_type, color = color, size = 10, interval = 15, linewidth = 3)

save_file_name = os.path.join(files_name, 'DESTINY.pkl')
with open(save_file_name, 'rb') as input:
    log = pickle.load(input)
    curve_type = line_stype[1]
    color = 'royalblue'
    plot_consensus_error(log, curve_type, color = color, size = 10, interval = 15, linewidth = 3)


large_font = 10
plt.rc('xtick', labelsize = 20)    # fontsize of the x and y labels
plt.xticks(fontsize = large_font)
plt.rc('ytick', labelsize = 20)
plt.yticks(fontsize = large_font)

plt.rc('legend', fontsize = 15)    # legend fontsize

plt.legend()
graph_type, weighted_rule, prob = log.graph

if not os.path.isdir("Results/Figures"):
    os.makedirs("Results/Figures")
save_path = 'Results/Figures'
#filename = 'PCA_shape_' + str(log.data_shape) + '_nodes_' + str(log.size) + '_p_' + str(prob)+ '_'+ graph_type +'_'+ weighted_rule  +'.pdf'
filename = 'Test_DESTINY_cons.pdf'
plt.tight_layout()
plt.savefig(os.path.join(save_path, filename), format = 'pdf', dpi = 3000)
plt.show()